package com.catalpa.commons.utils;

import com.baomidou.mybatisplus.annotation.DbType;
import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.generator.FastAutoGenerator;
import com.baomidou.mybatisplus.generator.config.*;
import com.baomidou.mybatisplus.generator.config.builder.CustomFile;
import com.baomidou.mybatisplus.generator.config.po.TableInfo;
import com.baomidou.mybatisplus.generator.config.rules.DateType;
import com.baomidou.mybatisplus.generator.config.rules.NamingStrategy;
import com.baomidou.mybatisplus.generator.engine.VelocityTemplateEngine;
import com.catalpa.commons.dto.GeneratorDto;
import jakarta.validation.constraints.NotNull;

import java.io.File;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author zjwu
 * 代码生成器主方法
 */
public class GeneratorUtil {

    private final GeneratorDto generatorDto;

    public GeneratorUtil(GeneratorDto generatorDto) {
        this.generatorDto = generatorDto;
    }

    /**
     * 代码生成方法
     */
    public void generate() {
        FastAutoGenerator.create(dataSourceConfig())
                .globalConfig(this::globalConfig)
                .packageConfig(this::packageConfig)
                .strategyConfig(this::strategyConfig)
                .injectionConfig(this::injectionConfig)
                .templateConfig(this::templateConfig)
                .templateEngine(new EnhanceFreemarkerTemplateEngine())
                .execute();
    }

    /**
     * 模版配置
     */
    private void templateConfig(TemplateConfig.Builder builder) {
        builder
                // 实体类
                .entity("/templates/entity.java")
                // 控制类
                .controller(("/templates/controller.java"))
                // 服务类
                .service("/templates/service.java")
                // 服务实现类
                .serviceImpl("/templates/serviceImpl.java")
                // 映射
                .mapper("/templates/mapper.java")
                // 映射XML
                .xml("/templates/mapper.xml");
    }

    /**
     * 注入配置
     */
    private void injectionConfig(InjectionConfig.Builder builder) {
        Map<String, Object> dtoMap = new HashMap<>(2);
        // 传输类包名
        dtoMap.put("dtoPackage", generatorDto.getDto());
        // 传输类覆盖已生成文件
        dtoMap.put("dtoOverride", true);
        builder
                // 传输类自定义参数
                .customMap(dtoMap)
                // 传输类模版配置
                .customFile(Collections.singletonMap("Dto.java", "/templates/dto.java.vm"));
    }

    /**
     * 策略配置
     */
    private void strategyConfig(StrategyConfig.Builder builder) {
        builder
                // 增加表匹配(内存过滤)
                .addInclude(generatorDto.getTableName())
                // 实体类策略配置
                .entityBuilder()
                // 实体类：开启lombok模型
                .enableLombok()
                // 实体类：开启链式模型
                .enableChainModel()
                // 实体类：数据库表映射到实体的命名策略，默认下划线转驼峰命名:NamingStrategy.underline_to_camel
                .naming(NamingStrategy.underline_to_camel)
                // 实体类：表字段映射实体属性命名规则，默认null，不指定按照naming执行
                .columnNaming(NamingStrategy.underline_to_camel)
                // 实体类：添加全局主键类型
                .idType(IdType.ASSIGN_UUID)
                // 实体类：格式化文件名称
                .formatFileName("%s")
                // 映射层策略配置
                .mapperBuilder()
                // 映射层：启用 BaseResultMap 生成
                .enableBaseResultMap()
                // 映射层：启用 BaseColumnList
                .enableBaseColumnList()
                // 映射层：格式化 mapper 文件名称
                .formatMapperFileName("%sDao")
                // 映射层：格式化 xml 实现类文件名称
                .formatXmlFileName("%sMapper")
                // 服务层策略配置
                .serviceBuilder()
                // 服务层：格式化 service 接口文件名称
                .formatServiceFileName("I%sService")
                // 服务层：格式化 service 实现类文件名称
                .formatServiceImplFileName("%sServiceImpl")
                // 控制层策略配置
                .controllerBuilder()
                // 控制层：开启生成@RestController 控制器
                .enableRestStyle();
        if (generatorDto.getOverride()) {
            builder.entityBuilder().enableFileOverride();
        }
    }

    /**
     * 包配置
     */
    private void packageConfig(PackageConfig.Builder builder) {
        builder
                // 父包名
                .parent(generatorDto.getParent())
                // 实体类包名
                .entity(generatorDto.getEntity())
                // 控制类包名
                .controller(generatorDto.getController())
                // 服务类包名
                .service(generatorDto.getService())
                // 服务实现类包名
                .serviceImpl(generatorDto.getServiceImpl())
                // 映射类包名
                .mapper(generatorDto.getMapper())
                // 映射类XML包名
                .xml(generatorDto.getXml());
    }

    /**
     * 全局配置
     */
    private void globalConfig(GlobalConfig.Builder builder) {
        builder
                // 禁止打开输出目录
                .disableOpenDir()
                // 作者名称
                .author(generatorDto.getAuthor())
                // 启用swagger
                .enableSpringdoc()
                // 时间策略
                .dateType(DateType.TIME_PACK)
                // 注释日期
                .commentDate("yyyy-MM-dd")
                // 指定输出目录
                .outputDir(generatorDto.getOutputDir());
    }

    /**
     * 数据源配置
     */
    private DataSourceConfig.Builder dataSourceConfig() {
        String databaseUrl = "";
        // MySQL数据库支持
        if (DbType.getDbType(generatorDto.getDatabaseType()) == DbType.MYSQL) {
            databaseUrl = "jdbc:mysql://" + generatorDto.getIp() + ":" + generatorDto.getPort() + "/" + generatorDto.getDatabaseName() + "?&useSSL=true&useUnicode=true&characterEncoding=utf-8&serverTimezone=Asia/Shanghai";
        }
        return new DataSourceConfig.Builder(databaseUrl, generatorDto.getUserName(), generatorDto.getUserPassword());
    }

    /**
     * 重写Dto生成方法的路径
     */
    public static class EnhanceFreemarkerTemplateEngine extends VelocityTemplateEngine {
        public void outputCustomFile(@NotNull List<CustomFile> customFiles, @NotNull TableInfo tableInfo, @NotNull Map<String, Object> objectMap) {
            String entityName = tableInfo.getEntityName();
            String parentPath = this.getPathInfo(OutputFile.parent) + "/" + objectMap.get("dtoPackage");
            for (CustomFile file : customFiles) {
                String filePath = StringUtils.isNotBlank(file.getFilePath()) ? file.getFilePath() : parentPath;
                if (StringUtils.isNotBlank(file.getPackageName())) {
                    filePath = filePath + File.separator + file.getPackageName();
                    filePath = filePath.replaceAll("\\.", "\\" + File.separator);
                }
                String fileName = filePath + File.separator + entityName + file.getFileName();
                this.outputFile(new File(fileName), objectMap, file.getTemplatePath(), (boolean) objectMap.get("dtoOverride"));
            }
        }
    }
}
